import sys
import os
import unittest
from abc import ABC, abstractmethod

class IRomanConverter(ABC):
    @abstractmethod
    def int_to_roman(self, num: int) -> str:
        pass

class RomanConverter_v1(IRomanConverter):
    def int_to_roman(self, num: int) -> str:
        """Konvertiert eine Dezimalzahl in eine römische Zahl."""
        val = [
            1000, 900, 500, 400,
            100, 90, 50, 40,
            10, 9, 5, 4,
            1
        ]
        syb = [
            "M", "CM", "D", "CD",
            "C", "XC", "L", "XL",
            "X", "IX", "V", "IV",
            "I"
        ]
        roman_num = ''
        i = 0
        while num > 0:
            for _ in range(num // val[i]):
                roman_num += syb[i]
                num -= val[i]
            i += 1
        return roman_num


"""class RomanConverter(IRomanConverter):
    def roman_to_int(self, s: str) -> int:
        Konvertiert eine römische Zahl (String) in eine Dezimalzahl.
        roman_values = {
            'I': 1, 'V': 5, 'X': 10, 'L': 50,
            'C': 100, 'D': 500, 'M': 1000
        }

        total = 0
        prev_value = 0

        # Iteriere über die Zeichen der römischen Zahl von rechts nach links
        for char in reversed(s.upper()): 
            value = roman_values.get(char)
            if value is None:
                raise ValueError(f"Ungültiges Zeichen '{char}' in der römischen Zahl.")
            if value < prev_value:
                total -= value
            else:
                total += value
            prev_value = value

        return total"""

# Testfälle roman_to_int, str -> int

# 1. Einzelne römische Ziffern
# Eingabe: "I" → Erwartete Ausgabe: 1
# Eingabe: "V" → Erwartete Ausgabe: 5
# Eingabe: "X" → Erwartete Ausgabe: 10
# Eingabe: "L" → Erwartete Ausgabe: 50
# Eingabe: "C" → Erwartete Ausgabe: 100
# Eingabe: "D" → Erwartete Ausgabe: 500
# Eingabe: "M" → Erwartete Ausgabe: 1000

#2. Mehrere gleiche Ziffern hintereinander (einfache Addition)
# Eingabe: "II" → Erwartete Ausgabe: 2
# Eingabe: "XX" → Erwartete Ausgabe: 20
# Eingabe: "CC" → Erwartete Ausgabe: 200
# Eingabe: "MM" → Erwartete Ausgabe: 2000

#3. Subtraktive Notation
# Eingabe: "IV" → Erwartete Ausgabe: 4
# Eingabe: "IX" → Erwartete Ausgabe: 9
# Eingabe: "XL" → Erwartete Ausgabe: 40
# Eingabe: "XC" → Erwartete Ausgabe: 90
# Eingabe: "CD" → Erwartete Ausgabe: 400
# Eingabe: "CM" → Erwartete Ausgabe: 900

#4. Komplexe Zahlen
# Eingabe: "MCMXCIV" → Erwartete Ausgabe: 1994
#Eingabe: "XIV" → Erwartete Ausgabe: 14
#Eingabe: "CDXLIV" → Erwartete Ausgabe: 444


import sys
import os
import unittest

class TestRomanConverter(unittest.TestCase):
    def setUp(self):
        self.converter = RomanConverter()

    def test_single_digits(self):
        self.assertEqual(self.converter.roman_to_int("I"), 1)
        self.assertEqual(self.converter.roman_to_int("V"), 5)
        self.assertEqual(self.converter.roman_to_int("X"), 10)

    def test_multiple_digits(self):
        self.assertEqual(self.converter.roman_to_int("II"), 2)
        self.assertEqual(self.converter.roman_to_int("XX"), 20)
        self.assertEqual(self.converter.roman_to_int("VI"), 6)

    def test_subtractive_notation(self):
        self.assertEqual(self.converter.roman_to_int("IV"), 4)
        self.assertEqual(self.converter.roman_to_int("IX"), 9)
        self.assertEqual(self.converter.roman_to_int("XL"), 40)
        self.assertEqual(self.converter.roman_to_int("XC"), 90)

    def test_complex_numbers(self):
        self.assertEqual(self.converter.roman_to_int("MCMXCIV"), 1994)
        self.assertEqual(self.converter.roman_to_int("CDXLIV"), 444)

if __name__ == "__main__":
    unittest.main()